@@ -12,17 +12,15 @@ pub mod spec;
 use std::{
     fmt::Write,
     io::Write as IoWrite,
-    num::ParseIntError,
-    path::{Path, PathBuf},
+    path::Path,
     str::FromStr,
-    sync::{atomic::AtomicBool, Arc, Mutex},
+    sync::{
+        atomic::{AtomicBool, Ordering},
+        Arc, Mutex,
+    },
     time::{Duration, Instant},
 };
 
-use bellframe::{
-    place_not::{self, PnBlockParseError},
-    InvalidRowError,
-};
 use log::{log_enabled, LevelFilter};
 use monument::{Comp, Config, Progress, Query, QueryUpdate};
 use ordered_float::OrderedFloat;
@@ -40,12 +38,11 @@ pub fn init_logging(filter: LevelFilter) {
 }
 
 pub fn run(
-    input_file: &Path,
+    source: Source,
     debug_option: Option<DebugOption>,
-    queue_limit: usize,
-) -> Result<Option<QueryResult>, Error> {
-    let start_time = Instant::now();
-
+    config: &Config,
+    ctrl_c_behaviour: CtrlCBehaviour,
+) -> anyhow::Result<Option<QueryResult>> {
     /// If the user specifies a [`DebugPrint`] flag with e.g. `-D layout`, then debug print the
     /// corresponding value and exit.
     macro_rules! debug_print {
@@ -57,61 +54,97 @@ pub fn run(
         };
     }
 
+    let start_time = Instant::now();
+
     // Generate & debug print the TOML file specifying the search
-    let spec =
-        Spec::read_from_file(input_file).map_err(|e| Error::SpecFile(input_file.to_owned(), e))?;
+    let spec = Spec::from_source(source)?;
     debug_print!(Spec, spec);
 
     // Convert the `Spec` into a `Layout` and other data required for running a search
     log::debug!("Generating query");
-    let query = spec.lower(input_file)?;
+    let query = spec.lower(source)?;
     debug_print!(Query, query);
     debug_print!(Layout, &query.layout);
 
-    // Generate config
-    let mut config = Config {
-        queue_limit,
-        num_threads: Some(1),
-        ..Config::default()
-    };
-
-    // Run query and handle its debug output
-    let graph = query.unoptimised_graph();
+    // Optimise the graph(s)
+    let graph = query
+        .unoptimised_graph(config)
+        .map_err(|e| anyhow::Error::msg(graph_build_err_msg(e)))?;
     debug_print!(Graph, graph);
-    let optimised_graphs = query.optimise_graph(graph, &mut config);
+    let optimised_graphs = query.optimise_graph(graph, config);
     if debug_option == Some(DebugOption::StopBeforeSearch) {
         return Ok(None);
     }
 
-    // Print comps as they are generated
+    // Thread-safe data for the query engine
+    let abort_flag = Arc::new(AtomicBool::new(false));
     let comps = Arc::new(Mutex::new(Vec::<Comp>::new()));
-    let comps_for_closure = comps.clone();
     let mut update_logger = SingleLineProgressLogger::new();
+
+    // Run the search
+    if ctrl_c_behaviour == CtrlCBehaviour::RecoverComps {
+        let abort_flag = abort_flag.clone();
+        let handler = move || abort_flag.store(true, Ordering::Relaxed);
+        if let Err(e) = ctrlc::set_handler(handler) {
+            log::warn!("Error capturing ctrl-C: {}", e);
+        }
+    }
+    let on_find_comp = {
+        let comps = comps.clone();
+        move |update| {
+            if let Some(comp) = update_logger.log(update) {
+                comps.lock().unwrap().push(comp);
+            }
+        }
+    };
     Query::search(
         Arc::new(query),
         optimised_graphs,
-        &config,
-        move |update| {
-            if let Some(comp) = update_logger.log(update) {
-                comps_for_closure.lock().unwrap().push(comp);
-            }
-        },
-        // User can abort with ctrl-C, so we don't need to use the abort flag
-        Arc::new(AtomicBool::new(false)),
+        config,
+        on_find_comp,
+        abort_flag.clone(),
     );
 
+    // Recover and sort the compositions, then return the query
     let mut comps = comps.lock().unwrap().to_vec();
     comps.sort_by_key(|comp| OrderedFloat(comp.music_score()));
     Ok(Some(QueryResult {
         comps,
-        duration: Instant::now() - start_time,
+        duration: start_time.elapsed(),
+        aborted: abort_flag.load(Ordering::SeqCst),
     }))
 }
 
+/// The `Source` of the TOML that Monument should read.  In nearly all cases, this will be loaded
+/// from a [`Path`].  For the test runner it's useful to be able to run Monument on strings that
+/// aren't loaded from a specific file, so for this we have the [`Str`](Self::Str) variant.
+#[derive(Debug, Clone, Copy)]
+pub enum Source<'a> {
+    /// The TOML 'spec' file should be loaded from this path
+    Path(&'a Path),
+    /// The TOML should be read directly from a string
+    Str {
+        spec: &'a str,
+        music_file: Option<&'a str>,
+    },
+}
+
+/// How this query run should handle `Ctrl-C`.  This is usually
+/// [`RecoverComps`](Self::RecoverComps) when running as a stand-alone command and
+/// [`TerminateProcess`](Self::TerminateProcess) when running in the unit tests.
+#[derive(Debug, PartialEq, Eq)]
+pub enum CtrlCBehaviour {
+    /// Terminate the process instantly, without attempting to recover the compositions
+    TerminateProcess,
+    /// Capture the `Ctrl-C`, abort the search, and print the comps
+    RecoverComps,
+}
+
 #[derive(Debug, Clone)]
 pub struct QueryResult {
     pub comps: Vec<Comp>,
     pub duration: Duration,
+    pub aborted: bool,
 }
 
 impl QueryResult {
@@ -120,32 +153,18 @@ impl QueryResult {
         for c in &self.comps {
             println!("{}", c);
         }
-        println!("Search completed in {}", PrettyDuration(self.duration));
+        println!(
+            "{} compositions generated.  Search {} {}",
+            self.comps.len(),
+            match self.aborted {
+                true => "aborted after",
+                false => "completed in",
+            },
+            PrettyDuration(self.duration)
+        );
     }
 }
 
-/// The possible ways that a run of Monument could fail
-#[derive(Debug)]
-pub enum Error {
-    SpecFile(PathBuf, spec::TomlReadError),
-    MusicFile(PathBuf, spec::TomlReadError),
-
-    PartHeadParse(InvalidRowError),
-    // TODO: For too short inputs, suggest expanding with either bells or a `*`
-    ChMaskParse(String, bellframe::mask::ParseError),
-    ChPatternParse(String, bellframe::mask::ParseError),
-
-    NoMethods,
-    CcLibNotFound,
-    MethodNotFound { suggestions: Vec<String> },
-    MethodPnParse(PnBlockParseError),
-
-    CallPnParse(String, place_not::ParseError),
-    LeadLocationIndex(String, ParseIntError),
-
-    LayoutGen(monument::layout::new::Error),
-}
-
 /// What item should be debug printed
 #[derive(Debug, Clone, Copy, PartialEq, Eq)]
 pub enum DebugOption {
@@ -177,6 +196,20 @@ impl FromStr for DebugOption {
     }
 }
 
+////////////////////
+// ERROR MESSAGES //
+////////////////////
+
+fn graph_build_err_msg(e: monument::graph::BuildError) -> String {
+    use monument::graph::BuildError as BE;
+    match e {
+        BE::SizeLimit(limit) => format!(
+            "Graph size limit of {} nodes reached.  You can set it higher with `--graph-size-limit <n>`.",
+            limit
+        )
+    }
+}
+
 /////////////////////////
 // SINGLE LINE LOGGING //
 /////////////////////////
@@ -186,6 +219,7 @@ impl FromStr for DebugOption {
 struct SingleLineProgressLogger {
     last_progress: Progress,
     is_truncating_queue: bool,
+    is_aborting: bool,
     /// The number of characters in the last line we printed.  `UpdateLogger` will use this add
     /// enough spaces to the end of the next message to completely overwrite the last one
     last_line_length: usize,
@@ -196,6 +230,7 @@ impl SingleLineProgressLogger {
         Self {
             last_progress: Progress::START,
             is_truncating_queue: false,
+            is_aborting: false,
             last_line_length: 0,
         }
     }
@@ -223,7 +258,11 @@ impl SingleLineProgressLogger {
 
         let std_out = std::io::stdout();
         let mut std_out = std_out.lock();
-        write!(std_out, "{}\r", update_string).unwrap();
+        // We precede with a carriage return to make sure that we overwrite anything the user
+        // types (e.g. `^C`).  We don't do anything in the case that the user's input is longer
+        // than what we're writing - we'll just assume that no-one would be able to type that much
+        // in between updates (which happen many many times per second).
+        write!(std_out, "\r{}\r", update_string).unwrap();
         std_out.flush().unwrap(); // `std_out` won't flush by default without a newline
 
         comp
@@ -238,6 +277,10 @@ impl SingleLineProgressLogger {
                 self.is_truncating_queue = false;
             }
             QueryUpdate::TruncatingQueue => self.is_truncating_queue = true,
+            QueryUpdate::Aborting => {
+                self.is_truncating_queue = false;
+                self.is_aborting = true;
+            }
         }
         None
     }
@@ -254,9 +297,12 @@ impl SingleLineProgressLogger {
             p.max_length
         )
         .unwrap();
-        if self.is_truncating_queue {
-            buf.push_str(".  Truncating queue...");
-        }
+        buf.push_str(match (self.is_aborting, self.is_truncating_queue) {
+            (false, false) => "",
+            (true, false) => ".  Aborting...",
+            (false, true) => ".  Truncating queue...",
+            (true, true) => unreachable!("Must either be aborting or truncating queue"),
+        });
     }
 
     /// Add whitespace to the end of a string to make sure it will cover the last thing we printed.
