@@ -23,7 +23,7 @@ use monument::{
         Layout,
     },
     music::{MusicType, StrokeSet},
-    OptRange, Query,
+    CallType, CallVec, OptRange, Query,
 };
 use serde::Deserialize;
 
@@ -66,6 +66,9 @@ pub struct Spec {
     /// Bounds on how many rows of each method is allowed
     #[serde(default)]
     method_count: OptRange,
+    /// Score which is applied for every change of method.  Defaults to `0.0`
+    #[serde(default)]
+    splice_weight: f32,
 
     /* CALLS */
     /// Which calls should be used by default
@@ -183,40 +186,52 @@ impl Spec {
             anyhow::Error::msg(format!("Can't parse part head {:?}: {}", self.part_head, e))
         })?;
         let (start_indices, end_indices) = self.start_end_indices(&part_head);
+
+        // Convert the methods
+        let is_spliced = loaded_methods.len() > 1;
         let shorthands = monument::utils::default_shorthands(
             loaded_methods
                 .iter()
                 .map(|(m, common)| (m.title(), common.shorthand.as_deref())),
         );
+        let method_count = self.method_count.or_range(&default_method_count(
+            loaded_methods.len(),
+            &self.length.range,
+        ));
+        if is_spliced {
+            log::info!("method count range: {:?}", method_count);
+        }
         let mut methods = Vec::new();
+        #[allow(clippy::or_fun_call)] // `{start,end}_indices.as_deref()` is really cheap
         for ((method, common), shorthand) in loaded_methods.into_iter().zip(shorthands) {
             let override_ch_masks = common
                 .course_heads
                 .map(|chs| parse_ch_masks(&chs, stage))
                 .transpose()?;
-            #[allow(clippy::or_fun_call)] // `start_indices.as_deref()` is a really cheap operation
-            let start_indices = common.start_indices.as_deref().or(start_indices.as_deref());
-            #[allow(clippy::or_fun_call)]
-            let end_indices = common.end_indices.as_deref().or(end_indices.as_deref());
+            let count_range = common.count_range.or_range(&method_count);
+            if is_spliced && common.count_range.is_set() {
+                log::info!("count range for {:<2}: {:?}", shorthand, count_range);
+            }
             methods.push(layout::new::Method::new(
                 method,
-                calls.clone(),
                 override_ch_masks.unwrap_or_else(|| ch_masks.clone()),
                 shorthand,
-                start_indices,
-                end_indices,
+                count_range,
+                common.start_indices.as_deref().or(start_indices.as_deref()),
+                common.end_indices.as_deref().or(end_indices.as_deref()),
             ));
         }
 
-        let method_count_range =
-            method_count_range(methods.len(), &self.length.range, self.method_count);
-        if methods.len() > 1 {
-            log::info!("Method count range: {:?}", method_count_range);
-        }
         let ch_weights = self.ch_weights(stage, methods.iter().map(Deref::deref))?;
 
         // Generate the `Layout`
-        let layout = Layout::new(methods, self.splice_style, &part_head, self.leadwise)?;
+        let layout = Layout::new(
+            methods,
+            &calls,
+            self.splice_style,
+            &part_head,
+            self.leadwise,
+        )?;
 
         let music_types = self.music_types(source, stage)?;
         if music_types.is_empty() {
@@ -229,11 +244,12 @@ impl Spec {
             layout,
             len_range: self.length.range.clone(),
             num_comps: self.num_comps,
-            method_count_range,
             allow_false: self.allow_false,
 
+            calls: calls.into_iter().map(CallType::from).collect(),
             part_head,
             ch_weights,
+            splice_weight: self.splice_weight,
 
             music_types,
             start_stroke: self.start_stroke,
@@ -283,7 +299,8 @@ impl Spec {
         Ok(music_types)
     }
 
-    fn calls(&self, stage: Stage) -> anyhow::Result<Vec<Call>> {
+    fn calls(&self, stage: Stage) -> anyhow::Result<CallVec<Call>> {
+        // Generate a full set of calls
         let mut call_specs = self.base_calls.to_call_specs(
             stage,
             self.bobs_only,
@@ -294,7 +311,35 @@ impl Spec {
         for specific_call in &self.calls {
             call_specs.push(specific_call.to_call_spec(stage)?);
         }
-        Ok(call_specs)
+        // Check for duplicate debug or display symbols
+        Self::check_for_duplicate_call_names(&call_specs, "display", |call| &call.display_symbol)?;
+        Self::check_for_duplicate_call_names(&call_specs, "debug", |call| &call.debug_symbol)?;
+        Ok(CallVec::from(call_specs))
+    }
+
+    /// Check for duplicate calls.  Calls are a 'duplicate' if assign the same symbol at the same
+    /// lead location but to **different** place notations.  If they're the same, then Monument
+    /// will de-duplicate them.
+    fn check_for_duplicate_call_names<'calls>(
+        call_specs: &'calls [layout::new::Call],
+        symbol_name: &str,
+        get_symbol: impl Fn(&'calls layout::new::Call) -> &'calls str,
+    ) -> anyhow::Result<()> {
+        let sorted_calls = call_specs
+            .iter()
+            .map(|call| (get_symbol(call), &call.lead_location, &call.place_not))
+            .sorted_by_key(|&(sym, lead_loc, _pn)| (sym, lead_loc));
+        for ((sym1, lead_location1, pn1), (sym2, lead_location2, pn2)) in
+            sorted_calls.tuple_windows()
+        {
+            if sym1 == sym2 && lead_location1 == lead_location2 && pn1 != pn2 {
+                return Err(anyhow::Error::msg(format!(
+                    "Call {} symbol {:?} (at {:?}) is used for both {} and {}",
+                    symbol_name, sym1, lead_location1, pn1, pn2
+                )));
+            }
+        }
+        Ok(())
     }
 
     fn ch_mask_preset(&self, stage: Stage) -> anyhow::Result<CourseHeadMaskPreset> {
@@ -422,16 +467,11 @@ fn parse_ch_masks(ch_mask_strings: &[String], stage: Stage) -> anyhow::Result<Ve
 
 /// Determine a suitable default range in which method counts must lie, thus enforcing decent
 /// method balance.
-fn method_count_range(
-    num_methods: usize,
-    len_range: &Range<usize>,
-    user_range: OptRange,
-) -> Range<usize> {
+fn default_method_count(num_methods: usize, len_range: &Range<usize>) -> Range<usize> {
     let min_f32 = len_range.start as f32 / num_methods as f32 * (1.0 - METHOD_BALANCE_ALLOWANCE);
     let max_f32 = len_range.end as f32 / num_methods as f32 * (1.0 + METHOD_BALANCE_ALLOWANCE);
-    let min = user_range.min.unwrap_or(min_f32.floor() as usize);
-    let max = user_range.max.unwrap_or(max_f32.ceil() as usize);
-    min..max + 1 // + 1 because the `method_count` is an **inclusive** range
+    // + 1 because the `method_count` is an **inclusive** range
+    (min_f32.floor() as usize)..(max_f32.ceil() as usize + 1)
 }
 
 ///////////////
@@ -469,6 +509,10 @@ pub enum MethodSpec {
 #[serde(deny_unknown_fields)]
 pub struct MethodCommon {
     shorthand: Option<String>,
+
+    /// Optional override for method count range
+    #[serde(default, rename = "count")]
+    count_range: OptRange,
     /// The inputs to this map are numerical strings representing sub-lead indices, and the
     /// outputs are the lead location names
     lead_locations: Option<HashMap<String, String>>,
